
/*++
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
Copyright (c) 1997, 1998, 1999  Microsoft Corporation
--*/
/*

   Interactive TV Link Checksum Calculator Sample Code

   This is a sample C program that illustrates the calculation of
   the checksum field for EIA-746a TV Links.

   Usage:
      TVLink '<http://test.com/>[name:foo]'
   Returns: 
      chksum = 02ee
      string = <http://test.com/>[name:foo][02ee]

   In addition, each line sent to stdin will be repeated to stdout 
   with the calculated checksum appended to the end.

*/

#include <stdio.h>

static unsigned long
lcsum(unsigned char *buf, unsigned long nbytes) {
  unsigned long    sum;
  unsigned long     nwords;

  nwords = nbytes >> 1;
  sum = 0;
  while (nwords-- > 0) {
    sum += *buf++ << 8;
    sum += *buf++;
  }
  
  if (nbytes & 1) {
    sum += *buf << 8;
  }
  return sum;
}

static unsigned short
eac(unsigned long sum) {
  unsigned short csum;
  
  csum = (unsigned short)(sum >> 16);
        
  while(csum !=0) {
    sum = csum + (sum & 0xffff);
        csum = (unsigned short)(sum >> 16);
  }
                  
  return (unsigned short)(sum & 0xffff);
}
                                
static unsigned short
cksum(unsigned char *buf, unsigned long nbytes) {
  unsigned short result = ~eac(lcsum(buf, nbytes));
  return result;
}

unsigned short
checksum(unsigned char *buf, unsigned long len) {
  unsigned short result = eac(lcsum(buf, len));
  return result;
}

static void
usage( char *name)
{

  printf("%s <data> [chksum]\n",name);

  exit(1);

}

int
main(int argc, char *argv[])
{
  char *ptr;
  unsigned long sum;
  unsigned short chksum;
  unsigned char* buffer;
  unsigned short length;
  unsigned char line[1024];
  
  if (argc < 2) {
        if (gets(line) == NULL) 
        usage(argv[0]);

        do {
                sum = lcsum(line, strlen(line));
                chksum = ~eac(sum);
                printf( "%s[%04x]\n", line, chksum);
        } while (gets(line) != NULL);
        
  }
  
  if (argc == 2) {
        length = strlen(argv[1]);
        
        buffer = (unsigned char *)malloc(length);
        strcpy(buffer, argv[1]);
        
    sum = lcsum(buffer, strlen(argv[1]));
    chksum = ~eac(sum);
    printf ("chksum = %04x\n", chksum);
        printf ("string = %s[%04x]\n", buffer, chksum);
  }
 
  if (argc == 3) {
    unsigned short value;
        unsigned short correct; 
    if (strlen(argv[2]) != 4)
      usage(argv[0]);

        length = strlen(argv[1]);
        
        buffer = (unsigned char *)malloc(length);
        strcpy(buffer, argv[1]);
         
    value = (unsigned short)strtoul(argv[2], (char**)NULL, 16);
    
        sum = value;
    sum += lcsum(buffer, length);
    chksum = ~eac(sum);
    
        sum = lcsum(buffer, length);
        correct = ~eac(sum);
        
    if (chksum == 0) {
      printf("chksum of \"%s\" matches!\n", buffer);
    } else
      printf("chksum of \"%s\" failed!\n Correct value is: %04x\n", buffer, correct);
  }

}
